@@ -1,12 +1,13 @@
 package parser
 
 import (
+	"bytes"
 	_ "embed"
 	"errors"
+	"fmt"
 	"strings"
 	"testing"
 
-	"github.com/google/go-cmp/cmp"
 	"github.com/joerdav/xc/models"
 )
 
@@ -16,8 +17,30 @@ var s string
 //go:embed testdata/notasks.md
 var e string
 
+func assertTask(t *testing.T, expected, actual models.Task) {
+	t.Helper()
+	if expected.Name != actual.Name {
+		t.Fatalf("name want=%q got=%q", expected.Name, actual.Name)
+	}
+	if strings.Join(expected.Description, ",") != strings.Join(actual.Description, ",") {
+		t.Fatalf("description want=%v got=%v", expected.Description, actual.Description)
+	}
+	if expected.Script != actual.Script {
+		t.Fatalf("script want=%q got=%q", expected.Script, actual.Script)
+	}
+	if expected.Dir != actual.Dir {
+		t.Fatalf("dir want=%q got=%q", expected.Dir, actual.Dir)
+	}
+	if strings.Join(expected.DependsOn, ",") != strings.Join(actual.DependsOn, ",") {
+		t.Fatalf("requires want=%v got=%v", expected.DependsOn, actual.DependsOn)
+	}
+	if strings.Join(expected.Inputs, ",") != strings.Join(actual.Inputs, ",") {
+		t.Fatalf("inputs want=%v got=%v", expected.Inputs, actual.Inputs)
+	}
+}
+
 func TestParseFile(t *testing.T) {
-	p, err := NewParser(strings.NewReader(s))
+	p, err := NewParser(strings.NewReader(s), "Tasks")
 	if err != nil {
 		t.Fatalf("unexpected error: %v", err)
 	}
@@ -49,18 +72,64 @@ echo "Hello, world2!"
 			DependsOn:   []string{"list", "list2"},
 		},
 	}
-	if diff := cmp.Diff(expected, result); diff != "" {
-		t.Errorf("tasks does not match expected: %s", diff)
+	if len(result) != len(expected) {
+		t.Fatalf("want %d tasks got %d", len(expected), len(result))
+	}
+	for i := range result {
+		assertTask(t, expected[i], result[i])
 	}
 }
 
 func TestParseFileNoTasks(t *testing.T) {
-	_, err := NewParser(strings.NewReader(e))
-	if !errors.Is(err, ErrNoTasksTitle) {
+	_, err := NewParser(strings.NewReader(e), "tasks")
+	if !errors.Is(err, ErrNoTasksHeading) {
 		t.Fatalf("expected error %v got: %v", "no Tasks section found", err)
 	}
 }
 
+func TestMultipleDirs(t *testing.T) {
+	p, _ := NewParser(strings.NewReader("dir: some dir"), "tasks")
+	p.currTask.Dir = "an existing dir"
+	_, err := p.parseAttribute()
+	if err == nil {
+		t.Fatal("expected error got nil")
+	}
+}
+
+func TestCommandlessTask(t *testing.T) {
+	p, _ := NewParser(strings.NewReader(`
+# Tasks
+## a task
+## another task
+`), "tasks")
+	_, err := p.parseTask()
+	if err == nil {
+		t.Fatal("expected error got nil")
+	}
+}
+
+func TestUnTerminatedCodeBlock(t *testing.T) {
+	p, _ := NewParser(strings.NewReader(`
+# Tasks
+## a task
+`+"```"+`
+some code
+`), "tasks")
+	_, err := p.parseTask()
+	if err == nil {
+		t.Fatal("expected error got nil")
+	}
+}
+
+func TestMultipleCodeBlocks(t *testing.T) {
+	p, _ := NewParser(strings.NewReader("```\ncode\n```"), "tasks")
+	p.currTask.Script = "an existing script"
+	err := p.parseCodeBlock()
+	if err == nil {
+		t.Fatal("expected error got nil")
+	}
+}
+
 func TestParseAttribute(t *testing.T) {
 	tests := []struct {
 		name            string
@@ -170,26 +239,60 @@ func TestParseAttribute(t *testing.T) {
 	for _, tt := range tests {
 		tt := tt
 		t.Run(tt.name, func(t *testing.T) {
-			p, _ := NewParser(strings.NewReader(tt.in))
+			p, _ := NewParser(strings.NewReader(tt.in), "tasks")
 			ok, err := p.parseAttribute()
 			if err != nil {
-				t.Error(err)
+				t.Fatal(err)
 			}
 			if ok == tt.expectNotOk {
-				t.Errorf("ok=%v want=%v", ok, !tt.expectNotOk)
+				t.Fatalf("ok=%v want=%v", ok, !tt.expectNotOk)
 			}
 			if tt.expectEnv != "" && p.currTask.Env[0] != tt.expectEnv {
-				t.Errorf("Env[0]=%s, want=%s", p.currTask.Env[0], tt.expectEnv)
+				t.Fatalf("Env[0]=%s, want=%s", p.currTask.Env[0], tt.expectEnv)
 			}
 			if tt.expectDependsOn != "" && p.currTask.DependsOn[0] != tt.expectDependsOn {
-				t.Errorf("DependsOn[0]=%s, want=%s", p.currTask.DependsOn[0], tt.expectDependsOn)
+				t.Fatalf("DependsOn[0]=%s, want=%s", p.currTask.DependsOn[0], tt.expectDependsOn)
 			}
 			if tt.expectInputs != "" && p.currTask.Inputs[0] != tt.expectInputs {
-				t.Errorf("Inputs[0]=%s, want=%s", p.currTask.Inputs[0], tt.expectInputs)
+				t.Fatalf("Inputs[0]=%s, want=%s", p.currTask.Inputs[0], tt.expectInputs)
 			}
 			if tt.expectDir != "" && p.currTask.Dir != tt.expectDir {
-				t.Errorf("Dir=%s, want=%s", p.currTask.Dir, tt.expectDir)
+				t.Fatalf("Dir=%s, want=%s", p.currTask.Dir, tt.expectDir)
 			}
 		})
 	}
 }
+
+func BenchmarkParse10_000Tasks(b *testing.B) {
+	var buf bytes.Buffer
+	buf.WriteString(`
+## Tasks
+		`)
+	for i := 0; i < 100; i++ {
+		buf.WriteString(`
+### task-` + fmt.Sprint(i) + `
+
+Print a message
+
+Requires: list, list2
+
+` + "Env: `somevar=val`" + `
+Inputs: FOO, BAR
+
+` + "```" + `
+echo "Hello, world!"
+echo "Hello, world2!"
+` + "```")
+	}
+	file := buf.String()
+	for i := 0; i < b.N; i++ {
+		p, err := NewParser(strings.NewReader(file), "tasks")
+		if err != nil {
+			b.Fatal(err)
+		}
+		_, err = p.Parse()
+		if err != nil {
+			b.Fatal(err)
+		}
+	}
+}
